@@ -2,6 +2,7 @@ use super::range_wrapper::RangeInclusiveStartWrapper;
 use crate::std_ext::*;
 use alloc::collections::BTreeMap;
 use core::fmt::{self, Debug};
+use core::iter::FromIterator;
 use core::marker::PhantomData;
 use core::ops::RangeInclusive;
 use core::prelude::v1::*;
@@ -473,6 +474,28 @@ where
     }
 }
 
+pub struct IntoIter<K, V> {
+    inner: alloc::collections::btree_map::IntoIter<RangeInclusiveStartWrapper<K>, V>,
+}
+impl<K, V> IntoIterator for RangeInclusiveMap<K, V> {
+    type Item = (RangeInclusive<K>, V);
+    type IntoIter = IntoIter<K, V>;
+    fn into_iter(self) -> Self::IntoIter {
+        IntoIter {
+            inner: self.btm.into_iter(),
+        }
+    }
+}
+impl<K, V> Iterator for IntoIter<K, V> {
+    type Item = (RangeInclusive<K>, V);
+    fn next(&mut self) -> Option<(RangeInclusive<K>, V)> {
+        self.inner.next().map(|(by_start, v)| (by_start.range, v))
+    }
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        self.inner.size_hint()
+    }
+}
+
 // We can't just derive this automatically, because that would
 // expose irrelevant (and private) implementation details.
 // Instead implement it in the same way that the underlying BTreeMap does.
@@ -486,6 +509,30 @@ where
     }
 }
 
+impl<K, V> FromIterator<(RangeInclusive<K>, V)> for RangeInclusiveMap<K, V>
+where
+    K: Ord + Clone + StepLite,
+    V: Eq + Clone,
+{
+    fn from_iter<T: IntoIterator<Item = (RangeInclusive<K>, V)>>(iter: T) -> Self {
+        let mut range_map = RangeInclusiveMap::new();
+        range_map.extend(iter);
+        range_map
+    }
+}
+
+impl<K, V> Extend<(RangeInclusive<K>, V)> for RangeInclusiveMap<K, V>
+where
+    K: Ord + Clone + StepLite,
+    V: Eq + Clone,
+{
+    fn extend<T: IntoIterator<Item = (RangeInclusive<K>, V)>>(&mut self, iter: T) {
+        iter.into_iter().for_each(move |(k, v)| {
+            self.insert(k, v);
+        })
+    }
+}
+
 pub struct Gaps<'a, K, V, StepFnsT> {
     /// Would be redundant, but we need an extra flag to
     /// avoid overflowing when dealing with inclusive ranges.
