@@ -2,6 +2,7 @@ use super::range_wrapper::RangeStartWrapper;
 use crate::std_ext::*;
 use alloc::collections::BTreeMap;
 use core::fmt::{self, Debug};
+use core::iter::FromIterator;
 use core::ops::Range;
 use core::prelude::v1::*;
 
@@ -380,6 +381,28 @@ where
     }
 }
 
+pub struct IntoIter<K, V> {
+    inner: alloc::collections::btree_map::IntoIter<RangeStartWrapper<K>, V>,
+}
+impl<K, V> IntoIterator for RangeMap<K, V> {
+    type Item = (Range<K>, V);
+    type IntoIter = IntoIter<K, V>;
+    fn into_iter(self) -> Self::IntoIter {
+        IntoIter {
+            inner: self.btm.into_iter(),
+        }
+    }
+}
+impl<K, V> Iterator for IntoIter<K, V> {
+    type Item = (Range<K>, V);
+    fn next(&mut self) -> Option<(Range<K>, V)> {
+        self.inner.next().map(|(by_start, v)| (by_start.range, v))
+    }
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        self.inner.size_hint()
+    }
+}
+
 // We can't just derive this automatically, because that would
 // expose irrelevant (and private) implementation details.
 // Instead implement it in the same way that the underlying BTreeMap does.
@@ -393,6 +416,30 @@ where
     }
 }
 
+impl<K, V> FromIterator<(Range<K>, V)> for RangeMap<K, V>
+where
+    K: Ord + Clone,
+    V: Eq + Clone,
+{
+    fn from_iter<T: IntoIterator<Item = (Range<K>, V)>>(iter: T) -> Self {
+        let mut range_map = RangeMap::new();
+        range_map.extend(iter);
+        range_map
+    }
+}
+
+impl<K, V> Extend<(Range<K>, V)> for RangeMap<K, V>
+where
+    K: Ord + Clone,
+    V: Eq + Clone,
+{
+    fn extend<T: IntoIterator<Item = (Range<K>, V)>>(&mut self, iter: T) {
+        iter.into_iter().for_each(move |(k, v)| {
+            self.insert(k, v);
+        })
+    }
+}
+
 pub struct Gaps<'a, K, V> {
     outer_range: &'a Range<K>,
     keys: core::iter::Peekable<alloc::collections::btree_map::Keys<'a, RangeStartWrapper<K>, V>>,
@@ -1026,4 +1073,23 @@ mod tests {
         map.insert(8..9, ());
         assert_eq!(format!("{:?}", map), "{2..5: (), 6..7: (), 8..9: ()}");
     }
+
+    // Iterator Tests
+
+    #[test]
+    fn into_iter_matches_iter() {
+        // Just use vec since that's the same implementation we'd expect
+        let mut range_map: RangeMap<u32, bool> = RangeMap::new();
+        range_map.insert(1..3, false);
+        range_map.insert(3..5, true);
+
+        let cloned = range_map.to_vec();
+        let consumed = range_map.into_iter().collect::<Vec<_>>();
+
+        // Correct value
+        assert_eq!(cloned, vec![(1..3, false), (3..5, true)]);
+
+        // Equality
+        assert_eq!(cloned, consumed);
+    }
 }
